import pandas as pd
import csv
import os

class Engine:
    def __init__(self, output_folder='/usr/home/shumeeshumee/domains/shumeeshumee.usermd.net/public_html/others/Sales_raport'):#/usr/home/shumeeshumee/domains/shumeeshumee.usermd.net/public_html/others/Sales_raport
        self.column_mapping = {
            "Lp.": "x",
            "Nr zamówienia": "ID ZAMÓWIENIA",
            "Nr zamówienia zewnętrzny": "x",
            "Platforma": "POCHODZENIE",
            "Waluta": "WALUTA",
            "Data zamówienia": "DATA", #drop10znakow done
            "Status zamówienia": "STATUS ZAMÓWIENIA",
            "Status płatności": "x",
            "Sposób płatności": "x",
            "Email klienta": "x",
            "Nazwa klienta": "x",
            "Telefon klienta": "TELEFON",
            "Nazwa płatnika": "x",
            "Nr VAT podatnika": "x",
            "Ulica płatnika": "x",
            "Miasto płatnika": "x",
            "Kod pocztowy płatnika": "x",
            "Kraj płatnika": "x",
            "Nazwa dostawy": "x",
            "Ulica dostawy": "x",
            "Miasto dostawy": "x",
            "Kod pocztowy dostawy": "x",
            "Kraj dostawy": "EMAIL", #"PL""CZ" ZAMIEN NA POLSKA LUB CZECHY done
            "Kwota netto": "x",
            "Kwota brutto": "x",
            "Nazwa wysyłki": "SPOSÓB WYSYŁKI",
            "Kwota wysyłki": "KOSZT WYSYŁKI",
            "SKU produktu": "ID PRODUKTU", # STWORZ POLE Z TAKIMI SAMYMI WARTOSCIAMI O NAZWIE "SKU" done
            "EAN produktu": "EAN",
            "Nazwa produktu": "NAZWA PRODUKTU",
            "Ilość produktu": "ILOŚĆ",
            "Cena netto produktu": "x",
            "Cena brutto produktu": "CENA",
            "Wartość VAT": "x",
            "Stawka VAT": "x"
        }
        #"NR NADAWCZY" PUSTA WEDŁUG KOLEJNOŚCI "NAZWA MAGAZYNU" done
        self.df = None
        self.output_folder = output_folder

    def load_data(self, input_file):
        self.df = pd.read_csv(input_file, delimiter=';')

    def map_column_names(self):
        self.df.rename(columns=self.column_mapping, inplace=True)

    def remove_mapped_columns(self):
        self.df = self.df.drop(columns=['x'])
    
    def remove_empty_lines(self, input_file):
        with open(input_file, 'r', newline='') as infile:
            reader = csv.reader(infile, delimiter=';')
            rows = [row for row in reader if any(field.strip() for field in row)]

        with open(input_file, 'w', newline='') as outfile:
            writer = csv.writer(outfile, delimiter=';')
            writer.writerows(rows)
    
    def order_columns(self):
        column_order = [
        'ID ZAMÓWIENIA','DATA','POCHODZENIE',
        'NAZWISKO','TELEFON','EMAIL','NAZWA PRODUKTU',
        'ID PRODUKTU','SKU','EAN','ILOŚĆ','CENA','WALUTA',
        'KOSZT WYSYŁKI','SPOSÓB WYSYŁKI','NR NADAWCZY','STATUS ZAMÓWIENIA','Nazwa magazynu'
        ]

        self.df = self.df[column_order]
    
    def save_data(self, output_file):
        output_path = os.path.join(self.output_folder, output_file)
        self.df.to_csv(output_path, index=False, sep=';')
        return output_path
    
    def add_additional_columns(self):
        address_column = [col for col in self.df.columns if col == 'STATUS ZAMÓWIENIA']

        if address_column:
            address_index = self.df.columns.get_loc(address_column[0])
            self.df.insert(address_index - 1, 'NR NADAWCZY', '')
            self.df.insert(address_index + 1, 'Nazwa magazynu', '')

    def duplicate_column(self):

         sku_column = [col for col in self.df.columns if col == 'ID PRODUKTU']

         if sku_column:
            sku_index = self.df.columns.get_loc(sku_column[0])
            self.df['SKU'] = self.df.iloc[:, sku_index]

    def duplicate_column_status(self):

         sku_column = [col for col in self.df.columns if col == 'POCHODZENIE']

         if sku_column:
            sku_index = self.df.columns.get_loc(sku_column[0])
            self.df['NAZWISKO'] = self.df.iloc[:, sku_index]

    def data_scraper(self):
        if 'DATA' in self.df.columns:
            self.df['DATA'] = self.df['DATA'].apply(lambda x: pd.to_datetime(x).strftime('%d.%m.%Y')[:10] if pd.notnull(x) else x)


    def country_dlc(self):
        if 'EMAIL' in self.df.columns:
            country_mapping = {"PL": "POLSKA", "CZ": "CZECHY"}
            self.df['EMAIL'] = self.df['EMAIL'].map(country_mapping)

    def main(self, input_file):
            self.remove_empty_lines(input_file)
            self.load_data(input_file)
            self.map_column_names()
            self.remove_mapped_columns()
            self.add_additional_columns()
            self.duplicate_column()
            self.duplicate_column_status()
            self.data_scraper()
            self.country_dlc()
            self.order_columns()
            output_file = f"zmieniony_{os.path.basename(input_file)}"
            output_path = self.save_data(output_file)
            return output_path

